/*
  Copyright (c) 2015, Matthias Schiffer <mschiffer@universe-factory.net>
  All rights reserved.

  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions are met:

    1. Redistributions of source code must retain the above copyright notice,
       this list of conditions and the following disclaimer.
    2. Redistributions in binary form must reproduce the above copyright notice,
       this list of conditions and the following disclaimer in the documentation
       and/or other materials provided with the distribution.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
  DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
  FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
  DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
  SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
  CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/


#pragma once

#include "rule.hpp"

#include <map>


namespace solar {

struct grammar_t {
	std::vector<rule_t> rules;

	std::map<std::string, std::string> nonterm_types;
	std::map<symbol_t, std::pair<std::string, std::string>> term_types;

	std::map<symbol_t, std::string> destructors;

	std::string header_block;
	std::string source_block;

	std::vector<std::pair<std::string, std::string>> extra_args;


	void add_rule(const rule_t &rule) {
		if (rules.empty()) {
			item_t init("");
			init.get_rhs().push_back(symbol_t::make_nonterm(rule.item.get_lhs()));
			rules.emplace_back(solar::rule_t {std::move(init), std::vector<std::pair<std::string, bool>>(), std::string()});
		}

		rules.push_back(rule);
	}

	const std::string & get_nonterm_type(const std::string &sym) const {
		static const std::string empty;

		auto it = nonterm_types.find(sym);
		if (it == nonterm_types.end())
			return empty;
		else
			return it->second;
	}

	const std::pair<std::string, std::string> & get_term_type(const symbol_t &sym) const {
		static const std::pair<std::string, std::string> empty;

		auto it = term_types.find(sym);
		if (it == term_types.end())
			return empty;
		else
			return it->second;
	}

	const std::string & get_type(const symbol_t &sym) const {
		switch (sym.get_type()) {
		case SYMBOL_TYPE_NONTERM:
			return get_nonterm_type(sym.get_value());

		default:
			return get_term_type(sym).first;
		}
	}
};

}
