/*
  Copyright (c) 2015, Matthias Schiffer <mschiffer@universe-factory.net>
  All rights reserved.

  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions are met:

    1. Redistributions of source code must retain the above copyright notice,
       this list of conditions and the following disclaimer.
    2. Redistributions in binary form must reproduce the above copyright notice,
       this list of conditions and the following disclaimer in the documentation
       and/or other materials provided with the distribution.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
  DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
  FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
  DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
  SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
  CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/


#pragma once

#include "generator.hpp"
#include "codegen.hpp"


namespace solar {

class output_common_t {
private:
	const generator_t *generator;

	std::string prefix;
	std::string token_prefix;

	std::map<std::string, unsigned> tokens;
	std::map<symbol_t, std::string> symbol_values;

protected:
	std::string prefixed(const std::string &str) const {
		return (prefix + str);
	}

	std::string symbol_case(const symbol_t &sym) const;

	std::string symbol_value(const symbol_t &sym) const {
		return symbol_values.at(sym);
	}

	const generator_t * get_generator() const {
		return generator;
	}

	const std::map<std::string, unsigned> & get_tokens() const {
		return tokens;
	}

	codegen_t::function_t sig_alloc() const {
		codegen_t::function_t sig(prefixed("context_t *"), prefixed("alloc"));
		sig.add_arg("void *(*alloc_func)(size_t)", "alloc_func", true);

		return sig;
	}

	codegen_t::function_t sig_free() const {
		codegen_t::function_t sig("void", prefixed("free"));
		sig.add_arg(prefixed("context_t *"), "parser");
		sig.add_arg("void (*free_func)(void *)", "free_func", true);

		return sig;
	}

	codegen_t::function_t sig_push() const {
		codegen_t::function_t sig("int", prefixed("push"));
		sig.add_arg(prefixed("context_t *"), "parser");
		sig.add_arg("int", "token");
		sig.add_arg("const " + prefixed("token_value_t *"), "value");

		for (const auto &arg : generator->get_grammar().extra_args)
			sig.add_arg(arg);

		return sig;
	}

public:
	output_common_t(const generator_t *generator0, const std::string &prefix0, const std::string &token_prefix0);
};

}
