/*
  Copyright (c) 2015, Matthias Schiffer <mschiffer@universe-factory.net>
  All rights reserved.

  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions are met:

    1. Redistributions of source code must retain the above copyright notice,
       this list of conditions and the following disclaimer.
    2. Redistributions in binary form must reproduce the above copyright notice,
       this list of conditions and the following disclaimer in the documentation
       and/or other materials provided with the distribution.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
  DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
  FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
  DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
  SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
  CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/


#include "lex.hpp"
#include "parse.hpp"
#include "generator_slr.hpp"
#include "output_slr.hpp"

#include <cstdio>
#include <cstdlib>
#include <iostream>
#include <memory>


namespace solar {

bool read_grammar(const char *filename, grammar_t *grammar) {
	FILE *file = fopen(filename, "r");
	if (!file) {
		std::fprintf(stderr, "unable to open file %s\n", filename);
		return false;
	}

	std::unique_ptr<lex_t> lexer(new lex_t(file));
	parse_context_t *parser = parse_alloc(std::malloc);
	int ret;

	do {
		int token;
		parse_token_value_t value;

		token = lexer->lex(&value);
		if (token < 0) {
			std::fprintf(stderr, "error at %s:%i:%i\n", filename,
				     lexer->get_location().first_line, lexer->get_location().first_column);
			return false;
		}

		ret = parse_push(parser, token, &value, grammar);
	} while (ret > 0);

	if (ret < 0) {
		std::fprintf(stderr, "error: parse error at %s:%i:%i\n", filename,
			     lexer->get_location().first_line, lexer->get_location().first_column);
		return false;
	}

	parse_free(parser, std::free);

	return true;
}

}


int main(int argc, char *argv[]) {
	using namespace solar;

	if (argc < 4) {
		std::fprintf(stderr, "Usage: solar INPUT SOURCE HEADER\n");
		return 1;
	}

	grammar_t grammar;
	if (!read_grammar(argv[1], &grammar))
		return 1;

	generator_slr_t generator(grammar);
	output_slr(&generator, argv[2], argv[3]);

	return 0;
}
